@@ -4,22 +4,30 @@
 from __future__ import annotations
 
 from math import ceil
-from typing import Any, Callable, Dict
+from typing import Any, Callable, Dict, List
 
 import vapoursynth as vs
-from vsutil import (depth, disallow_variable_format,
-                    disallow_variable_resolution, fallback, get_depth, get_w,
-                    get_y, join, plane, scale_value)
+from vsutil import (depth, fallback, get_depth, get_w, get_y, join, plane,
+                    scale_value)
 
-from . import kernels, util
+from .kernels import Bicubic, Catrom, Point
 from .scale import ssim_downsample
-from .util import scale_thresh
+from .util import check_variable, pick_repair, scale_thresh
 
 core = vs.core
 
 
-@disallow_variable_format
-@disallow_variable_resolution
+__all__: List[str] = [
+    'clamp_aa',
+    'taa',
+    'nnedi3', 'eedi3',
+    'nneedi3_clamp',
+    'transpose_aa',
+    'upscaled_sraa', 'sraa',
+    'based_aa'
+]
+
+
 def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, strength: float = 1) -> vs.VideoNode:
     """
     Clamp stronger AAs to weaker AAs.
@@ -34,7 +42,7 @@ def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, streng
 
     :return:         Clip with clamped anti-aliasing.
     """
-    assert src.format
+    check_variable(src, "clamp_aa")
 
     if src.format is None or weak.format is None or strong.format is None:
         raise ValueError("clamp_aa: 'Variable-format clips not supported'")
@@ -47,8 +55,6 @@ def clamp_aa(src: vs.VideoNode, weak: vs.VideoNode, strong: vs.VideoNode, streng
         else core.std.ShufflePlanes([clamp, src], planes=[0, 1, 2], colorfamily=vs.YUV)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode]) -> vs.VideoNode:
     """
     Perform transpose AA.
@@ -59,14 +65,15 @@ def taa(clip: vs.VideoNode, aafun: Callable[[vs.VideoNode], vs.VideoNode]) -> vs
 
     :return:       Antialiased clip
     """
+    check_variable(clip, "taa")
     assert clip.format
 
     y = get_y(clip)
 
     aa = aafun(y.std.Transpose())
-    aa = kernels.Catrom().scale(aa, width=clip.width, height=clip.width, shift=(0.5, 0))
+    aa = Catrom().scale(aa, width=clip.width, height=clip.width, shift=(0.5, 0))
     aa = aafun(aa.std.Transpose())
-    aa = kernels.Catrom().scale(aa, width=clip.width, height=clip.height, shift=(0.5, 0))
+    aa = Catrom().scale(aa, width=clip.width, height=clip.height, shift=(0.5, 0))
 
     return aa if clip.format.color_family == vs.GRAY \
         else core.std.ShufflePlanes([aa, clip], planes=[0, 1, 2], colorfamily=vs.YUV)
@@ -119,8 +126,6 @@ def eedi3(opencl: bool = False, **override: Any) -> Callable[[vs.VideoNode], vs.
     return _eedi3
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
                   mask: vs.VideoNode | None = None,
                   mthr: float = 0.25, opencl: bool = False) -> vs.VideoNode:
@@ -139,6 +144,7 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
 
     :return:                    Antialiased clip
     """
+    check_variable(clip, "nneedi3_clamp")
     assert clip.format
 
     y = get_y(clip)
@@ -149,8 +155,6 @@ def nneedi3_clamp(clip: vs.VideoNode, strength: float = 1,
     return merged if clip.format.color_family == vs.GRAY else core.std.ShufflePlanes([merged, clip], [0, 1, 2], vs.YUV)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def transpose_aa(clip: vs.VideoNode,
                  eedi3: bool = False,
                  rep: int = 13) -> vs.VideoNode:
@@ -174,6 +178,7 @@ def transpose_aa(clip: vs.VideoNode,
 
     :return:          Antialiased clip
     """
+    check_variable(clip, "transpose_aa")
     assert clip.format
 
     clip_y = get_y(clip)
@@ -184,9 +189,9 @@ def transpose_aa(clip: vs.VideoNode,
 
     def _taa(clip: vs.VideoNode) -> vs.VideoNode:
         aa = _aafun(clip.std.Transpose())
-        aa = kernels.Catrom().scale(aa, clip.height, clip.width, shift=(0.5, 0))
+        aa = Catrom().scale(aa, clip.height, clip.width, shift=(0.5, 0))
         aa = _aafun(aa.std.Transpose())
-        return kernels.Catrom().scale(aa, clip.width, clip.height, shift=(0.5, 0))
+        return Catrom().scale(aa, clip.width, clip.height, shift=(0.5, 0))
 
     def _csharp(flt: vs.VideoNode, clip: vs.VideoNode) -> vs.VideoNode:
         blur = core.std.Convolution(flt, [1] * 9)
@@ -194,40 +199,38 @@ def transpose_aa(clip: vs.VideoNode,
 
     aaclip = _taa(clip_y)
     aaclip = _csharp(aaclip, clip_y)
-    aaclip = util.pick_repair(clip_y)(aaclip, clip_y, rep)
+    aaclip = pick_repair(clip_y)(aaclip, clip_y, rep)
 
     return aaclip if clip.format.color_family is vs.GRAY else core.std.ShufflePlanes([aaclip, clip], [0, 1, 2], vs.YUV)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def _nnedi3_supersample(clip: vs.VideoNode, width: int, height: int, opencl: bool = False) -> vs.VideoNode:
+    check_variable(clip, "_nnedi3_supersample")
+
     nnargs: Dict[str, Any] = dict(field=0, dh=True, nsize=0, nns=4, qual=2)
     _nnedi3 = nnedi3(opencl=opencl, **nnargs)
     up_y = _nnedi3(get_y(clip))
-    up_y = kernels.Catrom().scale(up_y, width=up_y.width, height=height, shift=(0.5, 0))
+    up_y = Catrom().scale(up_y, width=up_y.width, height=height, shift=(0.5, 0))
     up_y = _nnedi3(up_y.std.Transpose())
-    up_y = kernels.Catrom().scale(up_y, width=up_y.width, height=width, shift=(0.5, 0))
-    return up_y
+    up_y = Catrom().scale(up_y, width=up_y.width, height=width, shift=(0.5, 0))
+    return up_y.std.Transpose()
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def _eedi3_singlerate(clip: vs.VideoNode) -> vs.VideoNode:
+    check_variable(clip, "_eedi3_singlerate")
+
     eeargs: Dict[str, Any] = dict(field=0, dh=False, alpha=0.2, beta=0.6, gamma=40, nrad=2, mdis=20)
     nnargs: Dict[str, Any] = dict(field=0, dh=False, nsize=0, nns=4, qual=2)
     y = get_y(clip)
     return eedi3(sclip=nnedi3(**nnargs)(y), **eeargs)(y)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def upscaled_sraa(clip: vs.VideoNode,
                   rfactor: float = 1.5,
                   width: int | None = None, height: int | None = None,
                   supersampler: Callable[[vs.VideoNode, int, int], vs.VideoNode] = _nnedi3_supersample,
                   downscaler: Callable[[vs.VideoNode, int, int], vs.VideoNode] | None
-                  = kernels.Bicubic(b=0, c=1/2).scale,
+                  = Bicubic(b=0, c=1/2).scale,
                   aafun: Callable[[vs.VideoNode], vs.VideoNode] = _eedi3_singlerate) -> vs.VideoNode:
     """
     A function that performs a supersampled single-rate AA to deal with heavy aliasing and broken-up lineart.
@@ -258,6 +261,7 @@ def upscaled_sraa(clip: vs.VideoNode,
 
     :return:                Antialiased clip
     """
+    check_variable(clip, "upscaled_sraa")
     assert clip.format
 
     luma = get_y(clip)
@@ -277,7 +281,7 @@ def upscaled_sraa(clip: vs.VideoNode,
 
     up_y = supersampler(luma, ssw, ssh)
 
-    aa_y = aafun(aafun(up_y).std.Transpose())
+    aa_y = aafun(aafun(up_y.std.Transpose()).std.Transpose())
 
     scaled = aa_y if downscaler is None else downscaler(aa_y, width, height)
 
@@ -287,8 +291,6 @@ def upscaled_sraa(clip: vs.VideoNode,
     return core.std.ShufflePlanes([scaled, clip], planes=[0, 1, 2], colorfamily=vs.YUV)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def based_aa(clip: vs.VideoNode, shader_file: str = "FSRCNNX_x2_56-16-4-1.glsl",
              rfactor: float = 2.0, tff: bool = True,
              mask_thr: float = 60, show_mask: bool = False,
@@ -339,21 +341,23 @@ def based_aa(clip: vs.VideoNode, shader_file: str = "FSRCNNX_x2_56-16-4-1.glsl",
         ow, oh = fallback(width, iw), fallback(height, ih)
 
         if (ow > iw and ow/iw != ow//iw) or (oh > ih and oh/ih != oh//ih):
-            mclip = kernels.Point().scale(mclip, iw * ceil(ow / iw), ih * ceil(oh / ih))
+            mclip = Point().scale(mclip, iw * ceil(ow / iw), ih * ceil(oh / ih))
         return core.fmtc.resample(mclip, ow, oh, kernel='box', fulls=1, fulld=1)
 
+    check_variable(clip, "based_aa")
     assert clip.format
 
     aaw = (round(clip.width * rfactor) + 1) & ~1
     aah = (round(clip.height * rfactor) + 1) & ~1
 
+    clip_y = get_y(clip)
+
     if not lmask:
         if mask_thr > 255:
             raise ValueError(f"based_aa: 'mask_thr must be equal to or lower than 255 (current: {mask_thr})'")
 
         mask_thr = scale_value(mask_thr, 8, get_depth(clip))
 
-        clip_y = get_y(clip)
         lmask = clip_y.std.Prewitt().std.Binarize(mask_thr).std.Maximum().std.BoxBlur(0, 1, 1, 1, 1)
 
     mclip_up = _resize_mclip(lmask, aaw, aah)
@@ -374,3 +378,7 @@ def based_aa(clip: vs.VideoNode, shader_file: str = "FSRCNNX_x2_56-16-4-1.glsl",
     if clip.format.num_planes == 1:
         return aa_merge
     return join([aa_merge, plane(clip, 1), plane(clip, 2)])
+
+
+# Aliases
+sraa = upscaled_sraa
