@@ -16,15 +16,26 @@ from typing import (Any, Callable, Dict, Iterable, Iterator, List, Literal,
                     Sequence, Set, Tuple, TypeVar, overload)
 
 import vapoursynth as vs
-from vsutil import disallow_variable_format, disallow_variable_resolution, get_w, get_subsampling, depth
+from vsutil import depth, get_subsampling, get_w
+from vsutil import split as split_planes
 
 from .dehardsub import hardsub_mask
 from .misc import get_matrix
 from .render import clip_async_render
-from .util import get_prop
+from .util import check_variable, get_prop
 
 core = vs.core
 
+__all__: List[str] = [
+    'Comparer', 'Stack', 'Interleave', 'Tile', 'Split',
+    'compare', 'comp', 'stack_compare', 'scomp',
+    'stack_planes',
+    'diff_hardsub_mask', 'diff',
+    'interleave', 'split', 'tile',
+    'stack_horizontal', 'stack_vertical',
+    'Direction'
+]
+
 T = TypeVar('T')
 
 
@@ -368,6 +379,9 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
         return core.resize.Bicubic(clip, format=vs.RGB24, matrix_in=get_matrix(clip),
                                    prefer_props=True, dither_type='error_diffusion')
 
+    check_variable(clip_a, "compare")
+    check_variable(clip_b, "compare")
+
     # Error handling
     if frames and len(frames) > clip_a.num_frames:
         raise ValueError("compare: 'More comparisons requested than frames available'")
@@ -395,8 +409,6 @@ def compare(clip_a: vs.VideoNode, clip_b: vs.VideoNode,
     return core.std.Interleave([frames_a, frames_b], mismatch=mismatch)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def stack_compare(*clips: vs.VideoNode,
                   make_diff: bool = True,
                   height: int = 288,
@@ -418,6 +430,7 @@ def stack_compare(*clips: vs.VideoNode,
 
     :return:          Clip with `clips` stacked
     """
+
     if not make_diff:
         warnings.warn("stack_compare has been deprecated in favour of `lvsfunc.comparison.Stack` "
                       "when you're not using `make_diff`", DeprecationWarning)
@@ -437,8 +450,6 @@ def stack_compare(*clips: vs.VideoNode,
     return Stack([Stack(resized).clip, diff], direction=Direction.VERTICAL).clip
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.VideoNode:
     """
     Stacks the planes of a clip.
@@ -451,17 +462,18 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
 
     :return:               Clip with stacked planes
     """
+    check_variable(clip, "stack_planes")
     assert clip.format
 
     if clip.format.num_planes != 3:
         raise ValueError("stack_planes: input clip must be in YUV or RGB planar format")
 
-    split_planes = split(clip)
+    yuv_planes = split_planes(clip)
 
     if clip.format.color_family == vs.ColorFamily.YUV:
-        planes: Dict[str, vs.VideoNode] = {'Y': split_planes[0], 'U': split_planes[1], 'V': split_planes[2]}
+        planes: Dict[str, vs.VideoNode] = {'Y': yuv_planes[0], 'U': yuv_planes[1], 'V': yuv_planes[2]}
     elif clip.format.color_family == vs.ColorFamily.RGB:
-        planes = {'R': split_planes[0], 'G': split_planes[1], 'B': split_planes[2]}
+        planes = {'R': yuv_planes[0], 'G': yuv_planes[1], 'B': yuv_planes[2]}
     else:
         raise ValueError(f"stack_planes: unexpected color family {clip.format.color_family.name}")
 
@@ -481,8 +493,6 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
         raise ValueError(f"stack_planes: unexpected subsampling {get_subsampling(clip)}")
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
     """
     Diff func for :py:func:`lvsfunc.comparison.diff` to use a hardsub mask.
@@ -493,7 +503,10 @@ def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.Vid
 
     :return:  Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`
     """
-    return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b))
+    check_variable(a, "stack_planes")
+    check_variable(b, "stack_planes")
+
+    return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b, **kwargs))
 
 
 @overload
@@ -520,8 +533,6 @@ def diff(*clips: vs.VideoNode,
     ...
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def diff(*clips: vs.VideoNode,
          thr: float = 72,
          height: int = 288,
@@ -574,12 +585,10 @@ def diff(*clips: vs.VideoNode,
     if thr >= 128:
         raise ValueError("diff: `thr` must be below 128")
 
-    if clips:
-        if any(c.format is None for c in clips):
-            raise ValueError("diff: variable-format clips not supported")
-    elif namedclips:
-        if any(nc.format is None for nc in namedclips.values()):
-            raise ValueError("diff: variable-format namedclips not supported")
+    if clips and any(c.format is None for c in clips):
+        raise ValueError("diff: variable-format clips not supported")
+    elif namedclips and any(nc.format is None for nc in namedclips.values()):
+        raise ValueError("diff: variable-format namedclips not supported")
 
     def _to_ranges(iterable: List[int]) -> Iterable[Tuple[int, int]]:
         iterable = sorted(set(iterable))
@@ -670,7 +679,6 @@ def interleave(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode
     return Interleave(clips if clips else namedclips).clip
 
 
-@disallow_variable_resolution
 def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience funciton for splitting clips along the x-axis and then stacking.
@@ -689,7 +697,6 @@ def split(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     return Split(clips if clips else namedclips, label_alignment=2).clip
 
 
-@disallow_variable_resolution
 def stack_horizontal(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience function for stacking clips horizontally.
@@ -705,7 +712,6 @@ def stack_horizontal(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Vid
     return Stack(clips if clips else namedclips).clip
 
 
-@disallow_variable_resolution
 def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience function for stacking clips vertically.
@@ -721,7 +727,6 @@ def stack_vertical(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.Video
     return Stack(clips if clips else namedclips, direction=Direction.VERTICAL).clip
 
 
-@disallow_variable_resolution
 def tile(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     """
     Small convenience function for tiling clips in a rectangular pattern.
@@ -739,3 +744,8 @@ def tile(*clips: vs.VideoNode, **namedclips: vs.VideoNode) -> vs.VideoNode:
     if clips and namedclips:
         raise ValueError("tile: positional clips and named keyword clips cannot both be given")
     return Tile(clips if clips else namedclips).clip
+
+
+# Aliases
+comp = compare
+scomp = stack_compare
