@@ -4,21 +4,23 @@
 from __future__ import annotations
 
 from functools import partial
-from typing import Any, Optional, Sequence, SupportsFloat, Tuple
+from typing import Any, List, Optional, Sequence, SupportsFloat, Tuple
 
 import vapoursynth as vs
-from vsutil import (Dither, depth, disallow_variable_format,
-                    disallow_variable_resolution, get_depth)
+from vsutil import Dither, depth, get_depth
 
-from .kernels import Catrom, Kernel
+from .kernels import Bicubic, Kernel, Point, get_kernel
 from .types import Matrix
-from .util import get_prop
+from .util import check_variable, get_prop
 
 core = vs.core
 
 
-@disallow_variable_format
-@disallow_variable_resolution
+__all__: List[str] = [
+    'vsdpir', 'autodb_dpir'
+]
+
+
 def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
                 strs: Sequence[float] = [30, 50, 75],
                 thrs: Sequence[Tuple[float, float, float]] = [(1.5, 2.0, 2.0), (3.0, 4.5, 4.5), (5.5, 7.0, 7.0)],
@@ -65,6 +67,7 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
 
     :return:                Deblocked clip
     """
+    check_variable(clip, "autodb_dpir")
     assert clip.format
 
     def _eval_db(n: int, f: Sequence[vs.VideoFrame],
@@ -134,11 +137,10 @@ def autodb_dpir(clip: vs.VideoNode, edgevalue: int = 24,
     return core.resize.Bicubic(debl, format=clip.format.id, matrix=matrix if not is_rgb else None)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 25, mode: str = 'deblock',
            matrix: Matrix | int | None = None, tiles: int | Tuple[int] | None = None,
-           cuda: bool = True, i444: bool = False, kernel: Kernel = Catrom(), **dpir_args: Any) -> vs.VideoNode:
+           cuda: bool = True, i444: bool = False, kernel: Kernel | str = Bicubic(b=0, c=0.5),
+           **dpir_args: Any) -> vs.VideoNode:
     """
     A simple vs-mlrt DPIR wrapper for convenience.
 
@@ -172,14 +174,18 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
     try:
         from vsmlrt import DPIR, Backend, DPIRModel
     except ModuleNotFoundError:
-        raise ModuleNotFoundError("deblock: 'vsmlrt is required to use deblocking function.'")
+        raise ModuleNotFoundError("vsdpir: 'vsmlrt is required to use vsdpir functions.'")
 
+    check_variable(clip, "vsdpir")
     assert clip.format
 
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     bit_depth = get_depth(clip)
     is_rgb, is_gray = (clip.format.color_family is f for f in (vs.RGB, vs.GRAY))
 
-    clip_32 = depth(clip, 32, dither_type=Dither.ERROR_DIFFUSION).std.Limiter()
+    clip_32 = depth(clip, 32, dither_type=Dither.ERROR_DIFFUSION)
 
     # TODO: Replace with a match-case?
     if mode.lower() == 'deblock':
@@ -195,7 +201,7 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
         dpir_args |= dict(backend=Backend.ORT_CUDA if cuda else Backend.OV_CPU)
 
     if is_rgb or is_gray:
-        return depth(DPIR(clip_32, **dpir_args), bit_depth)
+        return depth(DPIR(clip_32.std.Limiter(), **dpir_args), bit_depth)
 
     if matrix is None:
         matrix = get_prop(clip.get_frame(0), "_Matrix", int)
@@ -203,8 +209,20 @@ def vsdpir(clip: vs.VideoNode, strength: SupportsFloat | vs.VideoNode | None = 2
     targ_matrix = Matrix(matrix)
     targ_format = clip.format.replace(subsampling_w=0, subsampling_h=0) if i444 else clip.format
 
-    clip_rgb = kernel.resample(clip_32, vs.RGBS, matrix_in=targ_matrix)  # type:ignore[arg-type]
+    clip_rgb = kernel.resample(clip_32, vs.RGBS, matrix_in=targ_matrix).std.Limiter()  # type:ignore[arg-type]
+
+    mod_w, mod_h = clip_rgb.width % 8, clip_rgb.height % 8
+
+    if to_pad := any([mod_w, mod_h]):
+        d_width, d_height = clip.width + mod_w, clip.height + mod_h
+
+        clip_rgb = Point(src_width=d_width, src_height=d_height).scale(
+            clip_rgb, d_width, d_height, (-mod_h, -mod_w)
+        )
 
     run_dpir = DPIR(clip_rgb, **dpir_args)
 
+    if to_pad:
+        run_dpir = run_dpir.std.Crop(0, mod_w, mod_h, 0)
+
     return kernel.resample(run_dpir, targ_format, targ_matrix)  # type:ignore[arg-type]
