@@ -5,6 +5,7 @@
 from __future__ import annotations
 
 import os
+import sys
 import time
 import warnings
 from fractions import Fraction
@@ -13,18 +14,31 @@ from pathlib import Path
 from typing import Any, Dict, List
 
 import vapoursynth as vs
-from vsutil import (Dither, depth, disallow_variable_format,
-                    disallow_variable_resolution, get_depth, get_w, get_y)
+from vsutil import Dither, depth, get_depth, get_w, get_y, scale_value
 
 from .comparison import Direction, Stack
-from .kernels import BicubicDidee, Catrom, Kernel
+from .kernels import Bicubic, BicubicDidee, Catrom, Kernel, get_kernel
 from .render import get_render_progress
-from .util import force_mod, get_prop, pick_repair
+from .util import (check_variable, force_mod, get_neutral_value, get_prop,
+                   pick_repair)
 
 core = vs.core
 
+__all__: List[str] = [
+    'sivtc', 'SIVTC', 'tivtc_vfr', 'TIVTC_VFR',
+    'seek_cycle',
+    'deblend', 'decomb',
+    'descale_fields',
+    'fix_telecined_fades',
+    'ivtc_credits',
+    'vinverse'
+]
 
-def SIVTC(clip: vs.VideoNode, pattern: int = 0,
+main_file = os.path.realpath(sys.argv[0]) if sys.argv[0] else None
+main_file = f"{os.path.splitext(os.path.basename(str(main_file)))[0]}_"
+
+
+def sivtc(clip: vs.VideoNode, pattern: int = 0,
           tff: bool = True, decimate: bool = True) -> vs.VideoNode:
     """
     A very simple fieldmatching function.
@@ -67,7 +81,7 @@ def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) ->
 
     :return:                Viewing UI for standard telecining cycles
     """
-    if not (scale & (scale-1) == 0) and scale != 0 and scale != -1:
+    if (scale & (scale-1) != 0) and scale != 0 and scale != -1:
         raise ValueError("seek_cycle: 'scale must be a value that is the power of 2!'")
 
     # TODO: 60i checks and flags somehow? false positives gonna be a pain though
@@ -109,10 +123,10 @@ def seek_cycle(clip: vs.VideoNode, write_props: bool = True, scale: int = -1) ->
     return Stack([vert_pad, stack, vert_pad, stack_abcde], direction=Direction.VERTICAL).clip
 
 
-def TIVTC_VFR(clip: vs.VideoNode,
-              tfm_in: Path | str = ".ivtc/matches.txt",
-              tdec_in: Path | str = ".ivtc/metrics.txt",
-              timecodes_out: Path | str = ".ivtc/timecodes.txt",
+def tivtc_vfr(clip: vs.VideoNode,
+              tfm_in: Path | str = f".ivtc/{main_file}matches.txt",
+              tdec_in: Path | str = f".ivtc/{main_file}metrics.txt",
+              timecodes_out: Path | str = f".ivtc/{main_file}timecodes.txt",
               decimate: int | bool = True,
               tfm_args: Dict[str, Any] = {},
               tdecimate_args: Dict[str, Any] = {}) -> vs.VideoNode:
@@ -174,8 +188,8 @@ def TIVTC_VFR(clip: vs.VideoNode,
         'input': str(tdec_in), 'tfmIn': str(tfm_in), 'mkvOut': str(timecodes_out),
     }
 
-    tfm = clip.tivtc.TFM(**tfm_args) if not decimate == -1 else clip
-    return tfm.tivtc.TDecimate(**tdecimate_args) if not int(decimate) == 0 else tfm
+    tfm = clip.tivtc.TFM(**tfm_args) if decimate != -1 else clip
+    return tfm.tivtc.TDecimate(**tdecimate_args) if int(decimate) != 0 else tfm
 
 
 def deblend(clip: vs.VideoNode, start: int = 0,
@@ -231,16 +245,16 @@ def decomb(clip: vs.VideoNode,
            rep: int | None = None, show_mask: bool = False,
            tfm_args: Dict[str, Any] = {},
            tdec_args: Dict[str, Any] = {},
+           vinv_args: Dict[str, Any] = {},
            qtgmc_args: Dict[str, Any] = {}) -> vs.VideoNode:
     """
     A filter that performs relatively aggressive filtering to get rid of the combing on a interlaced/telecined source.
     Decimation can be disabled if the user wishes to decimate the clip themselves.
 
     Enabling vinverse will result in more aggressive decombing at the cost of potential detail loss.
-    Sharpen will perform a directional unsharpening. Direction can be set using `dir`.
     A reference clip can be passed with `ref`, which will be used by TFM to create the output frames.
 
-    Base function written by Midlifecrisis from the WEEB AUTISM server, and modified by LightArrowsEXE.
+    Base function written by Midlifecrisis from the WEEB AUTISM server, modified by LightArrowsEXE.
 
     Dependencies:
 
@@ -253,17 +267,16 @@ def decomb(clip: vs.VideoNode,
     :param mode:          Sets the matching mode or strategy to use for TFM
     :param decimate:      Decimate the video after deinterlacing (Default: True)
     :param vinv:          Use vinverse to get rid of additional combing (Default: False)
-    :param sharpen:       Unsharpen after deinterlacing (Default: False)
-    :param dir:           Directional vector. 'v' = Vertical, 'h' = Horizontal (Default: v)
     :param rep:           Repair mode for repairing the decombed clip using the original clip (Default: None)
     :param show_mask:     Return combmask
     :param tfm_args:      Arguments to pass to TFM
+    :param vinv_args:     Arguments to pass to vinverse
     :param qtgmc_args:    Arguments to pass to QTGMC
 
     :return:              Decombed and optionally decimated clip
     """
     try:
-        from havsfunc import QTGMC, Vinverse
+        from havsfunc import QTGMC
     except ModuleNotFoundError:
         raise ModuleNotFoundError("decomb: missing dependency 'havsfunc'")
 
@@ -293,14 +306,15 @@ def decomb(clip: vs.VideoNode,
 
     decombed = core.std.FrameEval(clip, partial(_pp, clip=clip, pp=qtgmc_merged), clip)
 
-    decombed = Vinverse(decombed) if vinv else decombed
+    decombed = vinverse(decombed, **vinv_args) if vinv else decombed
     decombed = pick_repair(clip)(decombed, clip, rep) if rep else decombed
     return core.tivtc.TDecimate(decombed, **tdec_args) if decimate else decombed
 
 
 def descale_fields(clip: vs.VideoNode, tff: bool = True,
                    width: int | None = None, height: int = 720,
-                   kernel: Kernel = Catrom(), src_top: float = 0.0) -> vs.VideoNode:
+                   kernel: Kernel | str = Bicubic(b=0, c=0.5),
+                   src_top: float = 0.0) -> vs.VideoNode:
     """
     Simple descaling wrapper for interwoven upscaled fields.
     This function also sets a frameprop with the kernel that was used.
@@ -317,7 +331,7 @@ def descale_fields(clip: vs.VideoNode, tff: bool = True,
     :param tff:         Top-field-first. `False` sets it to Bottom-Field-First
     :param width:       Native width. Will be automatically determined if set to `None`
     :param height:      Native height. Will be divided by two internally
-    :param kernel:      lvsfunc.Kernel object (default: Catrom)
+    :param kernel:      lvsfunc.Kernel object. This can also be a string (default: Catrom)
     :param src_top:     Shifts the clip vertically during the descaling
 
     :return:            Descaled GRAY clip
@@ -325,12 +339,15 @@ def descale_fields(clip: vs.VideoNode, tff: bool = True,
     height_field = int(height/2)
     width = width or get_w(height, clip.width/clip.height)
 
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     clip = clip.std.SetFieldBased(2-int(tff))
 
     sep = core.std.SeparateFields(get_y(clip))
     descaled = kernel.descale(sep, width, height_field, (src_top, 0))
     weave_y = core.std.DoubleWeave(descaled)
-    weave_y = weave_y.std.SetFrameProp('scaler', data=f'{kernel.__name__} (Fields)')  # type: ignore[attr-defined]
+    weave_y = weave_y.std.SetFrameProp('scaler', data=f'{kernel.__class__.__name__} (Fields)')
     return weave_y.std.SetFieldBased(0)[::2]
 
 
@@ -418,8 +435,6 @@ def fix_telecined_fades(clip: vs.VideoNode, tff: bool | int | None = None,
     return ftf
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
                  interlaced: bool = True, dec: bool | None = None,
                  bob_clip: vs.VideoNode | None = None, qtgmc_args: Dict[str, Any] = {}
@@ -457,6 +472,8 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
     except ModuleNotFoundError:
         raise ModuleNotFoundError("ivtc_credits: missing dependency 'havsfunc'")
 
+    check_variable(clip, "ivtc_credits")
+
     if clip.fps != Fraction(30000, 1001):
         raise ValueError("ivtc_credits: 'Your clip must have a framerate of 30000/1001!'")
 
@@ -579,3 +596,48 @@ def ivtc_credits(clip: vs.VideoNode, frame_ref: int, tff: bool | None = None,
         fix2 = c2.mv.FlowInter(super2, vect_b2, vect_f2).std.SelectEvery(4, [0, 2])
 
         return core.std.Interleave([fix1, fix2] if pattern == 0 else [fix2, fix1])
+
+
+def vinverse(clip: vs.VideoNode, sstr: float = 2.0,
+             amount: int = 128, scale: float = 1.5) -> vs.VideoNode:
+    """
+    A simple function to clean up residual combing after a deinterlacing pass.
+    This is Setsugen_no_ao's implementation, adopted into lvsfunc.
+
+    :param clip:    Input clip.
+    :param sstr:    Contrasharpening strength. Increase this if you find
+                    the decombing blurs the image a bit too much.
+    :param amount:  Maximum difference allowed between the original pixels and adjusted pixels.
+                    Scaled to input clip's depth. Set to 255 to effectively disable this.
+    :param scale:   Scale amount for vertical sharp * vertical blur.
+
+    :return:        Clip with residual combing largely removed
+    """
+    check_variable(clip, "vinverse")
+    assert clip.format
+
+    if amount > 255:
+        raise ValueError("vinverse: '`amount` may not be set higher than 255!'")
+
+    neutral = get_neutral_value(clip)
+
+    # Expression to find combing and seperate it from the rest of the clip
+    find_combs = clip.akarin.Expr(f'{neutral} n! x x 2 * x[0,-1] x[0,1] + + 4 / - n@ +')
+
+    # Expression to decomb it (creates blending)
+    decomb = core.akarin.Expr([find_combs, clip],
+                              f'{neutral} n! x 2 * x[0,-1] x[0,1] + + 4 / blur! y x blur@ - x n@ - * 0 < n@ x blur@ '
+                              ' - abs x n@ - abs < x blur@ - n@ + x ? ? - n@ +')
+
+    # Final expression to properly merge it and avoid creating too much damage
+    return core.akarin.Expr([clip, decomb],
+                            f'{neutral} n! {scale_value(amount, 8, clip.format.bits_per_sample)} a! y y y y 2 * y[0,-1]'
+                            f' y[0,1] + + 4 / - {sstr} * + y - n@ + sdiff! x y - n@ + diff! sdiff@ n@ - diff@ n@ - '
+                            f'* 0 < sdiff@ n@ - abs diff@ n@ - abs < sdiff@ diff@ ? n@ - {scale} * n@ + sdiff@ n@ '
+                            '- abs diff@ n@ - abs < sdiff@ diff@ ? ? n@ - + merge! x a@ + merge@ < x a@ + x a@ - '
+                            'merge@ > x a@ - merge@ ? ?')
+
+
+# Temporary aliases
+SIVTC = sivtc
+TIVTC_VFR = tivtc_vfr
