@@ -11,15 +11,19 @@
 from __future__ import annotations
 
 import vapoursynth as vs
-from vsutil import disallow_variable_resolution
+
+from typing import List
 
 from .kernels import Point
-from .util import force_mod
+from .util import force_mod, check_variable
 
 core = vs.core
 
+__all__: List[str] = [
+    'minecraftify'
+]
+
 
-@disallow_variable_resolution
 def minecraftify(clip: vs.VideoNode, div: float = 64.0, mod: int | None = None) -> vs.VideoNode:
     """
     Function that transforms your clip into a Minecraft.
@@ -31,6 +35,8 @@ def minecraftify(clip: vs.VideoNode, div: float = 64.0, mod: int | None = None)
 
     :return:        A Minecraft.
     """
+    check_variable(clip, "minecraftify")
+
     ow, oh = round(clip.width/div), round(clip.height/div)
 
     if mod is not None:
