@@ -5,22 +5,28 @@ from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from functools import partial
-from typing import Callable, List, Tuple
+from typing import Any, Callable, Dict, List, Tuple
 
 import vapoursynth as vs
 from vsutil import Range as CRange
-from vsutil import (depth, disallow_variable_format,
-                    disallow_variable_resolution, get_y, iterate, join, split)
+from vsutil import depth, get_y, iterate, join, split
 
 from . import util
-from .types import Position, Range, Size
-from .util import replace_ranges, scale_thresh, pick_removegrain
+from .types import Position, Range, Shapes, Size
+from .util import (check_variable, pick_removegrain, replace_ranges,
+                   scale_thresh)
 
 core = vs.core
 
 
-@disallow_variable_format
-@disallow_variable_resolution
+__all__: List[str] = [
+    'detail_mask', 'detail_mask_neo',
+    'halo_mask', 'range_mask',
+    'BoundingBox', 'DeferredMask',
+    'mt_xxpand_multi', 'minm', 'maxm'
+]
+
+
 def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
                 rad: int = 3, brz_a: float = 0.025, brz_b: float = 0.045) -> vs.VideoNode:
     """
@@ -45,6 +51,8 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
 
     :return:            Detail mask
     """
+    check_variable(clip, "detail_mask")
+
     brz_a = scale_thresh(brz_a, clip)
     brz_b = scale_thresh(brz_b, clip)
 
@@ -62,12 +70,10 @@ def detail_mask(clip: vs.VideoNode, sigma: float | None = None,
     return util.pick_removegrain(mask)(mask, 11)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
                     detail_brz: float = 0.05, lines_brz: float = 0.08,
                     blur_func: Callable[[vs.VideoNode, vs.VideoNode, float],
-                                        vs.VideoNode] = core.bilateral.Bilateral,
+                                        vs.VideoNode] | None = None,
                     edgemask_func: Callable[[vs.VideoNode], vs.VideoNode] = core.std.Prewitt,
                     rg_mode: int = 17) -> vs.VideoNode:
     """
@@ -85,13 +91,18 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
                             Will not binarize if set to 0.
     :param blur_func:       Blurring function used for the detail detection.
                             Must accept the following parameters: ``clip``, ``ref_clip``, ``sigma``.
+                            Uses `bilateral.Bilateral` by default.
     :param edgemask_func:   Edgemasking function used for the edge detection
     :param rg_mode:         Removegrain mode performed on the final output
 
     :return:                Detail mask
     """
+    check_variable(clip, "detail_mask_neo")
     assert clip.format
 
+    if not blur_func:
+        blur_func = core.bilateral.Bilateral
+
     detail_brz = scale_thresh(detail_brz, clip)
     lines_brz = scale_thresh(lines_brz, clip)
 
@@ -115,8 +126,6 @@ def detail_mask_neo(clip: vs.VideoNode, sigma: float = 1.0,
     return depth(rm_grain, clip.format.bits_per_sample)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def halo_mask(clip: vs.VideoNode, rad: int = 2,
               brz: float = 0.35,
               thmi: float = 0.315, thma: float = 0.5,
@@ -144,6 +153,8 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
 
     :return:                Halo mask
     """
+    check_variable(clip, "halo_mask")
+
     smax = scale_thresh(1.0, clip)
 
     thmi, thma, thlimi, thlima = (scale_thresh(t, clip, assume=8) for t in [thmi, thma, thlimi, thlima])
@@ -173,8 +184,6 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
     return core.std.Expr(mask, expr="x 2 *")
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
     """
     Min/max mask with separate luma/chroma radii.
@@ -195,7 +204,7 @@ def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
 
     :return:        Range mask
     """
-    assert clip.format
+    check_variable(clip, "range_mask")
 
     if radc == 0:
         clip = get_y(clip)
@@ -254,6 +263,8 @@ class BoundingBox():
 
         :return:    Square mask representing the bounding box.
         """
+        check_variable(ref, "get_mask")
+
         if ref.format is None:
             raise ValueError("BoundingBox: 'Variable-format clips not supported'")
 
@@ -325,8 +336,10 @@ class DeferredMask(ABC):
 
         :return:      Bounded mask
         """
-        if ref.format is None or clip.format is None:
-            raise ValueError("get_mask: 'Variable-format clips not supported'")
+        check_variable(clip, "get_mask")
+        check_variable(ref, "get_mask")
+        assert clip.format
+        assert ref.format
 
         if self.bound:
             bm = self.bound.get_mask(ref)
@@ -352,3 +365,51 @@ class DeferredMask(ABC):
     @abstractmethod
     def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode) -> vs.VideoNode:
         pass
+
+
+def mt_xxpand_multi(clip: vs.VideoNode,
+                    sw: float = 1, sh: float | None = None,
+                    mode: Shapes | int = Shapes.ELLIPSE,
+                    start: int = 0,
+                    m__imum: Callable[..., vs.VideoNode] = core.std.Maximum,
+                    planes: List[int] = [0, 1, 2],
+                    **m_params: Any) -> List[vs.VideoNode]:
+    """
+    Mask expanding/inpanding function written by Zastin.
+
+    Performs multiple Minimums/Maximums.
+    """
+    check_variable(clip, "mt_xxpand_multi")
+    assert clip.format
+
+    params: Dict[str, Any] = dict(planes=planes)
+    params |= m_params
+
+    sh = sh or sw
+
+    # TODO: Match case
+    if mode == Shapes.ELLIPSE:
+        coordinates = [[1] * 8, [0, 1, 0, 1, 1, 0, 1, 0],
+                       [0, 1, 0, 1, 1, 0, 1, 0]]
+    elif mode == Shapes.LOSANGE:
+        coordinates = [[0, 1, 0, 1, 1, 0, 1, 0]] * 3
+    else:
+        coordinates = [[1] * 8] * 3
+
+    clips = [clip]
+    end = int(min(sw, sh)) + start
+
+    for x in range(start, end):
+        clips += [m__imum(clips[-1], coordinates=coordinates[x % 3], **params)]
+
+    for x in range(end, int(end + sw - sh)):
+        clips += [m__imum(clips[-1], coordinates=[0, 0, 0, 1, 1, 0, 0, 0], **params)]
+
+    for x in range(end, int(end + sh - sw)):
+        clips += [m__imum(clips[-1], coordinates=[0, 1, 0, 0, 0, 0, 1, 0], **params)]
+
+    return clips
+
+
+maxm = partial(mt_xxpand_multi, m__imum=core.std.Maximum)
+minm = partial(mt_xxpand_multi, m__imum=core.std.Minimum)
