@@ -11,20 +11,33 @@ from functools import partial, wraps
 from typing import Any, Callable, Dict, List, Sequence, Tuple, TypeVar, cast
 
 import vapoursynth as vs
-from vsutil import (depth, disallow_variable_format,
-                    disallow_variable_resolution, get_depth, get_w, get_y,
-                    is_image, scale_value)
+from vsutil import depth, get_depth, get_w, get_y, is_image, scale_value
 
 from .kernels import Catrom
 from .mask import BoundingBox
 from .types import Matrix, Position, Range, Size
-from .util import get_prop, normalize_ranges
+from .util import check_variable, get_prop, normalize_ranges
 from .util import replace_ranges as _replace_ranges
 from .util import scale_thresh as _scale_thresh
 
 core = vs.core
 
 
+__all__: List[str] = [
+    'source', 'src',
+    'edgefixer', 'ef',
+    'get_matrix',
+    'shift_tint',
+    'limit_dark',
+    'wipe_row',
+    'load_bookmarks', 'frames_since_bookmark',
+    'allow_variable', 'chroma_injector',
+    'colored_clips',
+    'unsharpen',
+    'overlay_sign'
+]
+
+
 # List of containers that are better off being indexed externally
 annoying_formats = ['.iso', '.ts', '.vob']
 
@@ -63,7 +76,6 @@ def source(file: str, ref: vs.VideoNode | None = None,
 
     :return:                  Vapoursynth clip representing input file
     """
-
     # TODO: find a way to NOT have to rely on a million elif's
     if file.startswith('file:///'):
         file = file[8::]
@@ -98,8 +110,8 @@ def source(file: str, ref: vs.VideoNode | None = None,
             clip = core.ffms2.Source(file, **index_args)
 
     if ref:
-        if ref.format is None:
-            raise ValueError("source: 'Variable-format ref clips not supported.'")
+        check_variable(ref, "source")
+        assert ref.format
 
         clip = core.std.AssumeFPS(clip, fpsnum=ref.fps.numerator, fpsden=ref.fps.denominator)
         clip = core.resize.Bicubic(clip, width=ref.width, height=ref.height,
@@ -110,8 +122,6 @@ def source(file: str, ref: vs.VideoNode | None = None,
     return clip
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def edgefixer(clip: vs.VideoNode,
               left: int | List[int] | None = None,
               right: int | List[int] | None = None,
@@ -149,6 +159,8 @@ def edgefixer(clip: vs.VideoNode,
                   "and will likely be renamed. Please make sure to update your older scripts once it does.",
                   FutureWarning)
 
+    check_variable(clip, "edgefixer")
+
     if left is None:
         left = 0
     if right is None:
@@ -162,8 +174,6 @@ def edgefixer(clip: vs.VideoNode,
     return ef if full_range else core.std.Limiter(ef, 16.0, [235, 240])
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def get_matrix(clip: vs.VideoNode) -> int:
     """
     Helper function to get the matrix for a clip.
@@ -172,6 +182,7 @@ def get_matrix(clip: vs.VideoNode) -> int:
 
     :return:        Value representing a matrix
     """
+    check_variable(clip, "get_matrix")
     assert clip.format
 
     frame = clip.get_frame(0)
@@ -186,8 +197,6 @@ def get_matrix(clip: vs.VideoNode) -> int:
     return 9
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def shift_tint(clip: vs.VideoNode, values: int | Sequence[int] = 16) -> vs.VideoNode:
     """
     A function for forcibly adding pixel values to a clip.
@@ -209,6 +218,8 @@ def shift_tint(clip: vs.VideoNode, values: int | Sequence[int] = 16) -> vs.Video
     """
     val: Tuple[float, float, float]
 
+    check_variable(clip, "shift_tint")
+
     if isinstance(values, int):
         val = (values, values, values)
     elif len(values) == 2:
@@ -253,15 +264,14 @@ def limit_dark(clip: vs.VideoNode, filtered: vs.VideoNode,
             return clip if psa > threshold else filtered
 
     if threshold_range and threshold_range > threshold:
-        raise ValueError(f"limit_dark: '\"threshold_range\" ({threshold_range}) must be a lower value than \"threshold\" ({threshold})'")  # noqa: E501
+        raise ValueError(f"limit_dark: '\"threshold_range\" ({threshold_range}) must be "
+                         "a lower value than \"threshold\" ({threshold})'")
 
     avg = core.std.PlaneStats(clip)
     return core.std.FrameEval(clip, partial(_diff, clip=clip, filtered=filtered,
                                             threshold=threshold, threshold_range=threshold_range), avg)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def wipe_row(clip: vs.VideoNode,
              ref: vs.VideoNode | None = None,
              pos: Position | Tuple[int, int] = (1, 1),
@@ -281,6 +291,8 @@ def wipe_row(clip: vs.VideoNode,
 
     :return:               Clip with given rows or columns wiped
     """
+    check_variable(clip, "wipe_row")
+
     ref = ref or core.std.BlankClip(clip)
 
     if size is None:
@@ -500,8 +512,6 @@ def colored_clips(amount: int,
     return [core.std.BlankClip(color=color, **blank_clip_args) for color in rgb_color_list]
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def unsharpen(clip: vs.VideoNode, strength: float = 1.0, sigma: float = 1.5,
               prefilter: bool = True, prefilter_sigma: float = 0.75) -> vs.VideoNode:
     """
@@ -509,32 +519,37 @@ def unsharpen(clip: vs.VideoNode, strength: float = 1.0, sigma: float = 1.5,
     Performs one-dimensional sharpening as such: "Original + (Original - blurred) * Strength"
     It then merges back noise and detail that was prefiltered away,
 
-    This function also tries to mask haloing created by the unsharpening.
-    You can disable this by setting `mask` to False.
+    Negative values will blur instead. This can be useful for trying to undo sharpening.
 
     This function is not recommended for normal use,
     but may be useful as prefiltering for detail- or edgemasks.
 
     :param clip:                Input clip.
-    :param strength:            Amount to multiply blurred clip with original clip by
+    :param strength:            Amount to multiply blurred clip with original clip by.
+                                Negative values will blur the clip instead.
     :param sigma:               Sigma for the gaussian blur.
     :param prefilter:           Pre-denoising to prevent the unsharpening from picking up random noise.
     :param prefilter_sigma:     Strength for the pre-denoising.
-    :param mask:                Use a halo mask to reduce haloing.
     :param show_mask:           Show halo mask.
 
     :return:                    Unsharpened clip
     """
+    check_variable(clip, "unsharpen")
+    assert clip.format
+
     den = clip.dfttest.DFTTest(sigma=prefilter_sigma) if prefilter else clip
     diff = core.std.MakeDiff(clip, den)
 
+    expr: str | List[str] = f'x y - {strength} * x +'
+
+    if clip.format.color_family is not vs.GRAY:
+        expr = [str(expr), "", ""]  # mypy wtf?
+
     blurred_clip = core.bilateral.Gaussian(den, sigma=sigma)
-    unsharp = core.std.Expr(clips=[den, blurred_clip], expr=[f'x y - {strength} * x +', "", ""])
+    unsharp = core.std.Expr([den, blurred_clip], expr)
     return core.std.MergeDiff(unsharp, diff)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
                  frame_ranges: Range | List[Range] | None = None, fade_length: int = 0,
                  matrix: Matrix | int | None = None) -> vs.VideoNode:
@@ -570,6 +585,7 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
     except ModuleNotFoundError:
         raise ModuleNotFoundError("overlay_sign: 'missing dependency `kagefunc`'")
 
+    check_variable(clip, "overlay_sign")
     assert clip.format
 
     ov_type = type(overlay)
@@ -631,9 +647,11 @@ def overlay_sign(clip: vs.VideoNode, overlay: vs.VideoNode | str,
         return replace_ranges(clip, merge, frame_ranges)
 
 
-# compatibility since these were moved to util
+# Aliases
 replace_ranges = _replace_ranges
 scale_thresh = _scale_thresh
+ef = edgefixer
+src = source
 
 # TODO: Write function that only masks px of a certain color/threshold of colors.
 #       Think the magic wand tool in various image-editing programs.
