@@ -8,14 +8,13 @@ from functools import partial
 from typing import Any, Callable, Dict, List, Literal, NamedTuple, cast
 
 import vapoursynth as vs
-from vsutil import (depth, disallow_variable_format,
-                    disallow_variable_resolution, get_depth, get_w, get_y,
-                    iterate, join, plane)
+from vsutil import depth, get_depth, get_w, get_y, iterate, join, plane
 
-from . import kernels
-from .kernels import Bicubic, BicubicSharp, Catrom, Kernel, Spline36
+from .kernels import (Bicubic, BicubicSharp, Catrom, Kernel, Spline36,
+                      get_kernel)
 from .types import VSFunction
-from .util import get_coefs, get_prop, quick_resample
+from .util import (check_variable, get_coefs, get_prop, quick_resample,
+                   scale_thresh)
 
 try:
     from cytoolz import functoolz
@@ -28,6 +27,15 @@ except ModuleNotFoundError:
 core = vs.core
 
 
+__all__: List[str] = [
+    'descale', 'descale_detail_mask', 'reupscale',
+    'ssim_downsample', 'CURVES',
+    'gamma2linear', 'linear2gamma',
+    'comparative_descale', 'comparative_restore',
+    'mixed_rescale'
+]
+
+
 class Resolution(NamedTuple):
     """ Tuple representing a resolution. """
 
@@ -54,8 +62,12 @@ class ScaleAttempt(NamedTuple):
     """ The subtractive difference between the original and descaled frame. """
 
 
+CreditMask = Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode]
+CustomScaler = Callable[[vs.VideoNode, int, int], vs.VideoNode]
+
+
 def _transpose_shift(n: int, f: vs.VideoFrame, clip: vs.VideoNode,
-                     kernel: kernels.Kernel, caller: str) -> vs.VideoNode:
+                     kernel: Kernel, caller: str) -> vs.VideoNode:
     h = f.height
     w = f.width
     clip = kernel.scale(clip, w, h*2, (0.5, 0))
@@ -63,7 +75,7 @@ def _transpose_shift(n: int, f: vs.VideoFrame, clip: vs.VideoNode,
 
 
 def _perform_descale(resolution: Resolution, clip: vs.VideoNode,
-                     kernel: kernels.Kernel) -> ScaleAttempt:
+                     kernel: Kernel) -> ScaleAttempt:
     descaled = kernel.descale(clip, resolution.width, resolution.height) \
         .std.SetFrameProp('descaleResolution', intval=resolution.height)
     rescaled = kernel.scale(descaled, clip.width, clip.height)
@@ -94,7 +106,7 @@ def _select_descale(n: int, f: vs.VideoFrame | List[vs.VideoFrame],
 @functoolz.curry
 def reupscale(clip: vs.VideoNode,
               width: int | None = None, height: int = 1080,
-              kernel: kernels.Kernel = kernels.Bicubic(b=0, c=1/2),
+              kernel: Kernel | str = Bicubic(b=0, c=1/2),
               **kwargs: Any) -> vs.VideoNode:
     """
     A quick 'n easy wrapper used to re-upscale a clip descaled with descale using znedi3.
@@ -118,9 +130,12 @@ def reupscale(clip: vs.VideoNode,
     """
     width = width or get_w(height)
 
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     # Doubling and downscale to given "h"
-    znargs = dict(nsize=4, nns=4, qual=2, pscrn=2)
-    znargs.update(kwargs)
+    znargs: Dict[str, Any] = dict(nsize=4, nns=4, qual=2, pscrn=2)
+    znargs |= kwargs
 
     upsc = quick_resample(clip, partial(core.znedi3.nnedi3, field=0, dh=True, **znargs))
     upsc = core.std.FrameEval(upsc, partial(_transpose_shift, clip=upsc,
@@ -133,8 +148,6 @@ def reupscale(clip: vs.VideoNode,
 
 
 @functoolz.curry
-@disallow_variable_format
-@disallow_variable_resolution
 def descale_detail_mask(clip: vs.VideoNode, rescaled_clip: vs.VideoNode,
                         threshold: float = 0.05) -> vs.VideoNode:
     """
@@ -149,22 +162,23 @@ def descale_detail_mask(clip: vs.VideoNode, rescaled_clip: vs.VideoNode,
 
     :return:               Mask of lost detail
     """
+    check_variable(clip, "descale_detail_mask")
+    check_variable(rescaled_clip, "descale_detail_mask")
+
     mask = core.std.Expr([get_y(clip), get_y(rescaled_clip)], 'x y - abs') \
         .std.Binarize(threshold)
     mask = iterate(mask, core.std.Maximum, 4)
     return iterate(mask, core.std.Inflate, 2)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def descale(clip: vs.VideoNode,
             upscaler:
             Callable[[vs.VideoNode, int, int], vs.VideoNode] | None = reupscale,
             width: int | List[int] | None = None,
             height: int | List[int] = 720,
-            kernel: kernels.Kernel = kernels.Bicubic(b=0, c=1/2),
+            kernel: Kernel | str = Bicubic(b=0, c=1/2),
             threshold: float = 0.0,
-            mask: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] | None
+            mask: CreditMask | vs.VideoNode | None
             = descale_detail_mask, src_left: float = 0.0, src_top: float = 0.0,
             show_mask: bool = False) -> vs.VideoNode:
     """
@@ -195,13 +209,13 @@ def descale(clip: vs.VideoNode,
                                     Note that if upscaler is None, no upscaling will be performed
                                     and neither detail masking nor proper fractional descaling can be preformed.
                                     (Default: :py:func:`lvsfunc.scale.reupscale`)
-    :param width:                   Width to descale to (if None, auto-calculated)
+    :param width:                   Width(s) to descale to (if None, auto-calculated)
     :param height:                  Height(s) to descale to. List indicates multiple resolutions,
                                     the function will determine the best. (Default: 720)
-    :param kernel:                  Kernel used to descale (see :py:class:`lvsfunc.kernels.Kernel`,
-                                    (Default: kernels.Bicubic(b=0, c=1/2))
+    :param kernel:                  Kernel used to descale the clip. This can also be a string.
+                                    (see :py:class:`lvsfunc.kernels.Kernel`, (Default: kernels.Bicubic(b=0, c=1/2))
     :param threshold:               Error threshold for conditional descaling (Default: 0.0, always descale)
-    :param mask:                    Function used to mask detail. If ``None``, no masking.
+    :param mask:                    Function or mask clip used to mask detail. If ``None``, no masking.
                                     Function must accept a clip and a reupscaled clip and return a mask.
                                     (Default: :py:func:`lvsfunc.scale.descale_detail_mask`)
     :param src_left:                Horizontal shifting for fractional resolutions (Default: 0.0)
@@ -210,8 +224,12 @@ def descale(clip: vs.VideoNode,
 
     :return:                       Descaled and re-upscaled clip with float bitdepth
     """
+    check_variable(clip, "descale")
     assert clip.format
 
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     if type(height) is int:
         height = [height]
 
@@ -276,7 +294,8 @@ def descale(clip: vs.VideoNode,
         rescaled = kernel.scale(descaled, clip.width, clip.height,
                                 (src_left, src_top))
         rescaled = rescaled.resize.Point(format=clip.format.id)
-        dmask = mask(clip_y, rescaled)
+
+        dmask = mask if isinstance(mask, vs.VideoNode) else mask(clip_y, rescaled)
 
         if upscaler is None:
             dmask = core.resize.Spline36(dmask, upscaled.width, upscaled.height)
@@ -304,11 +323,9 @@ CURVES = Literal[
 ]
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int = 720,
                     smooth: float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
-                    kernel: Kernel = Catrom(), gamma: bool = False,
+                    kernel: Kernel | str = Catrom(), gamma: bool = False,
                     curve: CURVES = vs.TransferCharacteristics.TRANSFER_BT709,
                     sigmoid: bool = False, epsilon: float = 1e-6) -> vs.VideoNode:
     """
@@ -346,6 +363,11 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
 
     :return: Downsampled clip
     """
+    check_variable(clip, "ssim_downsample")
+
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     if isinstance(smooth, int):
         filter_func = partial(core.std.BoxBlur, hradius=smooth, vradius=smooth)
     elif isinstance(smooth, float):
@@ -381,15 +403,14 @@ def ssim_downsample(clip: vs.VideoNode, width: int | None = None, height: int =
     return d
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  epsilon: float = 1e-6) -> vs.VideoNode:
+    check_variable(clip, "gamma2linear")
     assert clip.format
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
-        raise ValueError('Only 32 bits float is allowed')
+        raise ValueError("gamma2linear: 'Only 32 bits float is allowed'")
 
     c = get_coefs(curve)
 
@@ -404,15 +425,14 @@ def gamma2linear(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
     return core.std.Expr(clip, expr).std.SetFrameProps(_Transfer=8)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def linear2gamma(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
                  sigmoid: bool = False, thr: float = 0.5, cont: float = 6.5,
                  ) -> vs.VideoNode:
+    check_variable(clip, "linear2gamma")
     assert clip.format
 
     if get_depth(clip) != 32 and clip.format.sample_type != vs.FLOAT:
-        raise ValueError('Only 32 bits float is allowed')
+        raise ValueError("linear2gamma: 'Only 32 bits float is allowed'")
 
     c = get_coefs(curve)
 
@@ -429,10 +449,8 @@ def linear2gamma(clip: vs.VideoNode, curve: CURVES, gcor: float = 1.0,
     return core.std.Expr(clip, expr).std.SetFrameProps(_Transfer=curve)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: int = 720,
-                        kernel: Kernel | None = None, thr: float = 5e-8) -> vs.VideoNode:
+                        kernel: Kernel | str | None = None, thr: float = 5e-8) -> vs.VideoNode:
     """
     Easy wrapper to descale to SharpBicubic and an additional kernel,
     compare them, and then pick one or the other.
@@ -442,7 +460,7 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
     :param clip:        Input clip
     :param width:       Width to descale to (if None, auto-calculated)
     :param height:      Descale height
-    :param kernel:      Kernel to compare BicubicSharp to (Default: Spline36 if None)
+    :param kernel:      Kernel to compare BicubicSharp to. This can also be a string (Default: Spline36 if None).
     :param thr:         Threshold for which kernel to pick
 
     :return:            Descaled clip
@@ -453,6 +471,11 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
 
         return sharp if other_diff - thr > sharp_diff else other
 
+    check_variable(clip, "comparative_descale")
+
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     bsharp = BicubicSharp()
     kernel = kernel or Spline36()
 
@@ -482,10 +505,8 @@ def comparative_descale(clip: vs.VideoNode, width: int | None = None, height: in
     return core.std.FrameEval(sharp, partial(_compare, sharp=sharp, other=other), [sharp_diff, other_diff])
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: int = 720,
-                        kernel: Kernel | None = None) -> vs.VideoNode:
+                        kernel: Kernel | str | None = None) -> vs.VideoNode:
     """
     Companion function to go with comparative_descale
     to reupscale the clip for descale detail masking.
@@ -494,7 +515,14 @@ def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: in
     :param width:       Width to upscale to (if None, auto-calculated)
     :param height:      Upscale height
     :param kernel:      Kernel to compare BicubicSharp to (Default: Spline36 if None)
+
+    :return:            Reupscaled clip
     """
+    check_variable(clip, "comparative_restore")
+
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+
     bsharp = BicubicSharp()
     kernel = kernel or Spline36()
 
@@ -514,6 +542,111 @@ def comparative_restore(clip: vs.VideoNode, width: int | None = None, height: in
     return core.std.FrameEval(sharp_up, partial(_compare, sharp_up=sharp_up, other_up=other_up), clip)
 
 
+def mixed_rescale(clip: vs.VideoNode, width: None | int = None, height: int = 720,
+                  kernel: Kernel | str = Bicubic(b=0, c=1/2),
+                  downscaler: CustomScaler | Kernel | str = ssim_downsample,
+                  credit_mask: CreditMask | vs.VideoNode | None = descale_detail_mask, mask_thr: float = 0.05,
+                  mix_strength: float = 0.25, show_mask: bool | int = False,
+                  nnedi3_args: Dict[str, Any] = {}, eedi3_args: Dict[str, Any] = {}) -> vs.VideoNode:
+    """
+    InsaneAA rewrite to be a much saner and easier to read function.
+    Descales and downscales the given clip and merges them together with a set strength.
+
+    This can be useful for dealing with a source that you can't accurately descale,
+    but you still want to force it. Not recommended to use it on everything, however.
+
+    A string can be passed instead of a Kernel object if you want to use that.
+    This gives you access to every kernel object in :py:func:`lvsfunc.kernels`.
+    For more information on what every kernel does, please refer to their documentation.
+
+    This is still a work in progress at the time of writing. Please use with care.
+
+    :param clip:            Input clip.
+    :param width:           Upscale width. If None, determine from `height` (default: None).
+    :param height:          Upscale height (Default: 720).
+    :param kernel:          Kernel used to descale the clip. This can also be a string.
+                            (see :py:class:`lvsfunc.kernels.Kernel`, Default: kernels.Bicubic(b=0, c=1/2) (Catrom)).
+    :param downscaler:      Kernel or custom scaler used to downscale the clip. This can also be a string.
+                            (see :py:class:`lvsfunc.kernels.Kernel`, Default: ``ssim_downsample``).
+    :param credit_mask:     Function or mask clip used to mask detail. If ``None``, no masking.
+                            Function must accept a clip and a reupscaled clip and return a mask.
+                            (Default: :py:func:`lvsfunc.scale.descale_detail_mask`).
+    :param mask_thr:        Binarization threshold for :py:func:`lvsfunc.scale.descale_detail_mask` (Default: 0.05).
+    :param mix_strength:    Merging strength between the descaled and downscaled clip.
+                            Stronger values will make the lineart look closer to the downscaled clip.
+                            This can get pretty dangerous very quickly if you use a sharp ``downscaler``!
+    :param show_mask:       Return the ``credit_mask``. If set to `2`, it will return the lineart mask instead.
+    :param nnedi3_args:     Additional args to pass to nnedi3.
+    :param eedi3_args:      Additional args to pass to eedi3.
+
+    :return:                Rescaled clip with a downscaled clip merged with it and credits masked.
+    """
+    check_variable(clip, "mixed_rescale")
+
+    assert clip.format
+
+    if not 0 <= mask_thr <= 1:
+        raise ValueError(f"mixed_rescale: '`mask_thr` must be between 0.0 and 1.0! Not {mask_thr}!'")
+
+    # Default settings set to match insaneAA as closely as reasonably possible
+    nnargs: Dict[str, Any] = dict(nsize=0, nns=4, qual=2, pscrn=1)
+    nnargs |= nnedi3_args
+
+    eediargs: Dict[str, Any] = dict(alpha=0.2, beta=0.25, gamma=1000, nrad=2, mdis=20)
+    eediargs |= eedi3_args
+
+    width = width or get_w(height, clip.width/clip.height, only_even=False)
+
+    if isinstance(kernel, str):
+        kernel = get_kernel(kernel)()
+    if isinstance(downscaler, str):
+        downscaler = get_kernel(downscaler)()
+
+    bits = get_depth(clip)
+    clip_y = get_y(clip)
+
+    line_mask = clip_y.std.Prewitt(scale=2).std.Maximum().std.Limiter()
+
+    descaled = kernel.descale(clip_y, width, height)
+    upscaled = kernel.scale(descaled, clip.width, clip.height)
+
+    if isinstance(downscaler, Kernel):
+        downscaled = downscaler.scale(clip_y, width, height)
+    else:
+        downscaled = depth(downscaler(clip_y, width, height), bits)
+        downscaler = Catrom()
+
+    merged = core.std.Expr([descaled, downscaled], f'x {mix_strength} * y 1 {mix_strength} - * +')
+
+    if isinstance(credit_mask, vs.VideoNode):
+        detail_mask = depth(credit_mask, get_depth(clip))
+    elif not credit_mask:
+        detail_mask = clip_y.std.BlankClip(length=1) * clip.num_frames
+    else:
+        detail_mask = descale_detail_mask(clip_y, upscaled, threshold=scale_thresh(mask_thr, clip))
+        detail_mask = iterate(detail_mask, core.std.Inflate, 2)
+        detail_mask = iterate(detail_mask, core.std.Maximum, 2).std.Limiter()
+
+    if show_mask == 2:
+        return line_mask
+    elif show_mask:
+        return detail_mask
+
+    sclip = merged.std.Transpose().znedi3.nnedi3(0, True, **nnargs) \
+        .std.Transpose().znedi3.nnedi3(0, True, **nnargs)
+    double = merged.std.Transpose().eedi3m.EEDI3(0, True, **eediargs) \
+        .std.Transpose().eedi3m.EEDI3(0, True, sclip=sclip, **eediargs)
+    rescaled = ssim_downsample(downscaler.shift(double, shift=(.5, .5)), height=clip.height)
+    rescaled = depth(rescaled, bits)
+
+    masked = core.std.MaskedMerge(rescaled, clip_y, detail_mask)
+    masked = core.std.MaskedMerge(clip_y, masked, line_mask)
+
+    if clip.format.num_planes == 1:
+        return masked
+    return core.std.ShufflePlanes([masked, clip], planes=[0, 1, 2], colorfamily=vs.YUV)
+
+
 # TODO: Write a function that checks every possible combination of B and C in bicubic
 #       and returns a list of the results. Possibly return all the frames in order of
 #       smallest difference to biggest. Not reliable, but maybe useful as starting point.
