@@ -4,10 +4,19 @@
 from __future__ import annotations
 
 from enum import IntEnum
-from typing import Any, NamedTuple, NoReturn, Optional, Protocol, Tuple, Union
+from typing import (Any, List, NamedTuple, NoReturn, Optional, Protocol, Tuple,
+                    Union)
 
 import vapoursynth as vs
 
+
+__all__: List[str] = [
+    'Range',
+    'Coordinate', 'Position', 'Size',
+    'Matrix', 'Coefs',
+    'VSFunction'
+]
+
 Range = Union[Optional[int], Tuple[Optional[int], Optional[int]]]
 
 
@@ -67,3 +76,9 @@ class Coefs(NamedTuple):
 class VSFunction(Protocol):
     def __call__(self, clip: vs.VideoNode, *args: Any, **kwargs: Any) -> vs.VideoNode:
         ...
+
+
+class Shapes(IntEnum):
+    RECTANGLE = 0
+    ELLIPSE = 1
+    LOSANGE = 2
