@@ -7,16 +7,27 @@ from __future__ import annotations
 from typing import Any, Callable, List, Sequence, Tuple, Type, TypeVar, Union
 
 import vapoursynth as vs
-from vsutil import (depth, disallow_variable_format,
-                    disallow_variable_resolution, get_subsampling)
+from vsutil import depth, get_depth, get_subsampling
 
 from .types import Coefs, Range
 
 core = vs.core
 
 
-@disallow_variable_format
-@disallow_variable_resolution
+__all__: List[str] = [
+    'quick_resample',
+    'pick_repair', 'pick_removegrain',
+    'get_prop',
+    'normalize_ranges', 'replace_ranges', 'rfs',
+    'scale_thresh', 'scale_peak',
+    'force_mod', 'padder',
+    'clamp_values',
+    'get_neutral_value', 'get_coefs',
+    'check_variable',
+    'VideoProp'
+]
+
+
 def quick_resample(clip: vs.VideoNode,
                    function: Callable[[vs.VideoNode], vs.VideoNode]
                    ) -> vs.VideoNode:
@@ -29,6 +40,7 @@ def quick_resample(clip: vs.VideoNode,
 
     :return:          Filtered clip in original depth
     """
+    check_variable(clip, "quick_resample")
     assert clip.format
 
     try:  # Excepts all generic because >plugin/script writers being consistent >_>
@@ -45,8 +57,6 @@ def quick_resample(clip: vs.VideoNode,
     return depth(filtered, clip.format.bits_per_sample)
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def pick_repair(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
     """
     Returns rgvs.Repair if the clip is 16 bit or lower, else rgsf.Repair.
@@ -58,12 +68,11 @@ def pick_repair(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
 
     :return:     Appropriate repair function for input clip's depth
     """
+    check_variable(clip, "pick_repair")
     assert clip.format
     return core.rgvs.Repair if clip.format.bits_per_sample < 32 else core.rgsf.Repair
 
 
-@disallow_variable_format
-@disallow_variable_resolution
 def pick_removegrain(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
     """
     Returns rgvs.RemoveGrain if the clip is 16 bit or lower, else rgsf.RemoveGrain.
@@ -77,6 +86,7 @@ def pick_removegrain(clip: vs.VideoNode) -> Callable[..., vs.VideoNode]:
 
     :return:     Appropriate RemoveGrain function for input clip's depth
     """
+    check_variable(clip, "pick_removegrain")
     assert clip.format
     return core.rgvs.RemoveGrain if clip.format.bits_per_sample < 32 else core.rgsf.RemoveGrain
 
@@ -151,10 +161,14 @@ def normalize_ranges(clip: vs.VideoNode, ranges: Range | List[Range]) -> List[Tu
 
 def replace_ranges(clip_a: vs.VideoNode,
                    clip_b: vs.VideoNode,
-                   ranges: Range | List[Range] | None) -> vs.VideoNode:
+                   ranges: Range | List[Range] | None,
+                   use_plugin: bool = True) -> vs.VideoNode:
     """
     A replacement for ReplaceFramesSimple that uses ints and tuples rather than a string.
-    Frame ranges are inclusive.
+    Frame ranges are inclusive. Optionally strings can still be used.
+
+    This function will try to call the `VapourSynth-RemapFrames` plugin before doing any of its own processing.
+    This should come with a speed boost, so it's recommended you install it.
 
     Examples with clips ``black`` and ``white`` of equal length:
 
@@ -165,32 +179,39 @@ def replace_ranges(clip_a: vs.VideoNode,
         * ``replace_ranges(black, white, [(200, -1)])``: replace 200 until the end with ``white``,
           leaving 1 frame of ``black``
 
-    :param clip_a:     Original clip
-    :param clip_b:     Replacement clip
-    :param ranges:     Ranges to replace clip_a (original clip) with clip_b (replacement clip).
+    Dependencies: VapourSynth-RemapFrames
 
-                       Integer values in the list indicate single frames,
+    :param clip_a:      Original clip
+    :param clip_b:      Replacement clip
+    :param ranges:      Ranges to replace clip_a (original clip) with clip_b (replacement clip).
 
-                       Tuple values indicate inclusive ranges.
+                        Integer values in the list indicate single frames,
 
-                       Negative integer values will be wrapped around based on clip_b's length.
+                        Tuple values indicate inclusive ranges.
 
-                       None values are context dependent:
+                        Negative integer values will be wrapped around based on clip_b's length.
 
-                           * None provided as sole value to ranges: no-op
-                           * Single None value in list: Last frame in clip_b
-                           * None as first value of tuple: 0
-                           * None as second value of tuple: Last frame in clip_b
+                        None values are context dependent:
+
+                            * None provided as sole value to ranges: no-op
+                            * Single None value in list: Last frame in clip_b
+                            * None as first value of tuple: 0
+                            * None as second value of tuple: Last frame in clip_b
+    :param use_plugin:  Use the ReplaceFramesSimple plugin for the rfs call.
 
     :return:           Clip with ranges from clip_a replaced with clip_b
     """
     if ranges is None:
         return clip_a
 
-    out = clip_a
-
     nranges = normalize_ranges(clip_b, ranges)
 
+    if use_plugin and hasattr(core, 'remap'):
+        return core.remap.ReplaceFramesSimple(clip_a, clip_b, mappings=' '  # type:ignore  # TODO: Fix stubs
+                                              .join(f'[{s} {e}]' for s, e in nranges))
+
+    out = clip_a
+
     for start, end in nranges:
         tmp = clip_b[start:end + 1]
         if start != 0:
@@ -213,8 +234,8 @@ def scale_thresh(thresh: float, clip: vs.VideoNode, assume: int | None = None) -
 
     :return:       Threshold scaled to [0, 2^clip.depth - 1] (if vs.INTEGER)
     """
-    if clip.format is None:
-        raise ValueError("scale_thresh: 'Variable-format clips not supported'")
+    check_variable(clip, "scale_thresh")
+    assert clip.format
 
     if thresh < 0:
         raise ValueError("scale_thresh: 'Thresholds must be positive.'")
@@ -247,8 +268,27 @@ def clamp_values(x: float, max_val: float, min_val: float) -> float:
     return min_val if x < min_val else max_val if x > max_val else x
 
 
-@disallow_variable_format
-@disallow_variable_resolution
+def get_neutral_value(clip: vs.VideoNode, chroma: bool = False) -> float:
+    """
+    Taken from vsutil. This isn't in any new versions yet, so mypy complains.
+    Will remove once vsutil does another version bump.
+
+    Returns the neutral value for the combination
+    of the plane type and bit depth/type of the clip as float.
+
+    :param clip:        Input clip.
+    :param chroma:      Whether to get luma or chroma plane value
+
+    :return:            Neutral value.
+    """
+    check_variable(clip, "get_neutral_value")
+    assert clip.format
+
+    is_float = clip.format.sample_type == vs.FLOAT
+
+    return (0. if chroma else 0.5) if is_float else float(1 << (get_depth(clip) - 1))
+
+
 def padder(clip: vs.VideoNode,
            left: int = 32, right: int = 32,
            top: int = 32, bottom: int = 32) -> vs.VideoNode:
@@ -264,6 +304,8 @@ def padder(clip: vs.VideoNode,
 
     :return:            Padded clip
     """
+    check_variable(clip, "padder")
+
     width = clip.width+left+right
     height = clip.height+top+bottom
 
@@ -292,3 +334,16 @@ def get_coefs(curve: vs.TransferCharacteristics) -> Coefs:
     }
 
     return gamma_linear_map[curve]
+
+
+def check_variable(clip: vs.VideoNode, function: str) -> None:
+    if clip.format is None:
+        raise ValueError(f"{function}: 'Variable-format clips not supported!'")
+    elif 0 in (clip.width, clip.height):
+        raise ValueError(f"{function}: 'Variable-resolution clips not supported!'")
+
+    assert clip.format
+
+
+# Aliases
+rfs = replace_ranges
